
from OpenGL.GL import *
import pygame
import sys
import glsupport

FORE_COLOR = (0, 0, 0)
BACK_COLOR = (255, 255, 255)
FONT_FILE = "../font/LinLibertine_Re-4.7.5.ttf"
FONT_SIZE = 24
LINE_HEIGHT = 1.5

def init():
    pygame.font.init()
    if not pygame.font.get_init(): 
        print "font subsystem not available"; 
        sys.exit(-1)
            

class Font (object):
    def __init__(self, size = None):
        if (size == None):
            size = FONT_SIZE
        self.size = size
        self.font = pygame.font.Font(FONT_FILE, size)
        
    def draw_multiline_string(self, lines, 
                              center = False, 
                              height_min = 0, 
                              height_max = 1):
        """draws multiple lines
        lines - array of strings
        center - draw left aligned column (if False) or centered (if True)
        height_min - minimum available height (between 0 and 1, bottom up)
        height_max - maximum available height (between 0 and 1, bottom up)
        
        text will be vertically centered between height_min and height_max"""
        SCREEN_WIDTH = glsupport.current_screen_width
        SCREEN_HEIGHT = glsupport.current_screen_height
        
        pix_lines = []
        max_width = 0
        for line in lines:
            pix_line = self._text_to_pixels(line)
            if pix_line.width > max_width: max_width = pix_line.width
            pix_lines.append(pix_line)
            
        pix_lines.reverse() # open GL is from bottom to top
        
        height_min *= SCREEN_HEIGHT
        height_max *= SCREEN_HEIGHT
        
        line_height_in_pixels = LINE_HEIGHT * self.size
        total_text_height = len(pix_lines) * line_height_in_pixels
        
        h_off = height_min + ((height_max - height_min - total_text_height) / 2.0)
        if not center: w_off = (SCREEN_WIDTH - max_width) / 2.0
        
        for pix_line in pix_lines:
            if center: w_off = (SCREEN_WIDTH - pix_line.width) / 2.0
            self._render_pixels(pix_line, w_off, h_off + (pix_line.height / 2.0))
            h_off += line_height_in_pixels
        
         
            
        
    def draw_string(self, string, x, y):
        """draws a single line string"""
        
        pixels = self._text_to_pixels(string)
        self._render_pixels(pixels, x, y)
        
        
    def _text_to_pixels(self, text):
        text = str(text)
        # Font.render(text, antialias, color, background=None)
        surface = self.font.render(text, True, FORE_COLOR, BACK_COLOR)
        return PixelArray(surface)
    
    def _render_pixels(self, pixels, x, y):
        SCREEN_WIDTH = glsupport.current_screen_width
        SCREEN_HEIGHT = glsupport.current_screen_height
        
        glDisable(GL_TEXTURE_2D)
        glViewport(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        glOrtho(0.0, 
                SCREEN_WIDTH - 1.0, 
                0.0, 
                SCREEN_HEIGHT - 1.0, 
                -1.0, 
                1.0)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        
        glPushMatrix()
        
        glRasterPos2f(x, y)
        glDrawPixels(pixels.width, pixels.height, 
                     GL_RGBA, GL_UNSIGNED_BYTE, 
                     pixels.pixels)
        
        glPopMatrix()
        
        
        glEnable(GL_TEXTURE_2D)
    

class PixelArray(object):
    def __init__(self, surface):
        self.pixels = pygame.image.tostring(surface, "RGBA", True)
        self.width, self.height = surface.get_size()