import pygame
import level
import texts
import glsupport
from globalconstants import *
import zoom

import font2


class TextState(object):
    def __init__(self, lines, hint = None):
        self.lines = lines
        self.hint = hint
        
    def entry(self, game):
        pygame.mixer.music.unpause()
    
    def do(self, game):
        glsupport.ClearScreen(1.0, 1.0, 1.0)
        if self.hint == None:
            game.small_font.draw_multiline_string(self.lines)
        else:
            game.small_font.draw_multiline_string(self.lines,
                                                  center=True,
                                                  height_min = 0.3)
            game.tiny_font.draw_multiline_string(self.hint, 
                                                 center = True,
                                                 height_max = 0.3)
        return game.input.lmb
    
    def exit(self, game):
        pass


class TitleState(object):
    def __init__(self, title, instructions):
        self.title = title
        self.instructions = instructions
        
    def entry(self, game):
        pygame.mixer.music.unpause()

    def do(self, game):
        glsupport.ClearScreen(1.0, 1.0, 1.0)
        game.large_font.draw_multiline_string(self.title, 
                                              center=True,
                                              height_min = 0.3)
        
        game.tiny_font.draw_multiline_string(self.instructions, 
                                              center=True, 
                                              height_max = 0.3)
        return game.input.lmb
    
    def exit(self, game):
        pass
        
    
class LevelState(object):
    def __init__(self, level_type):
        self.level_type = level_type
        
    def entry(self, game):
        pygame.mixer.music.pause()
        self.level = self.level_type()
        zoom.pan_in_to_default()
        
    def do(self, game):
        if game.input.rmb:
            pygame.mixer.music.unpause()
        else:
            pygame.mixer.music.pause()
        
        if game.input.wheel_up:
            zoom.viewport_zoom_in()
                
        if game.input.wheel_down:
            zoom.viewport_zoom_out()
        
        self.level.tick(game.input)
        # return True
        return self.level.is_done()
    
    def exit(self, game):
        glsupport.release_all_textures()


class Game(object):
    """Game Class."""
    
    def __init__(self):
        glsupport.InitializeOpenGl(SCREEN_INIT_WIDTH, SCREEN_INIT_HEIGHT)

        pygame.init()
        font2.init()
              
        self.large_font = font2.Font(80)
        self.small_font = font2.Font(24)
        self.tiny_font = font2.Font(14)
        
        pygame.mixer.init()
        pygame.mixer.music.load("../audio/hameln90v1_pseudostacc.ogg")
        pygame.mixer.music.play(-1)

        self.clock = pygame.time.Clock()
        
        self.input = Input()
        
        self._states = (
                       TitleState(texts.GAME_TITLE, texts.INSTRUCTIONS),
                       TextState(texts.INTRO_1_1, texts.HELP_1),
                       TextState(texts.INTRO_1_2),
                       LevelState(level.LevelOne),
                       TextState(texts.INTRO_2_1, texts.HELP_2),
                       TextState(texts.INTRO_2_2),
                       LevelState(level.LevelTwo),
                       TextState(texts.GAME_OVER_1),
                       TextState(texts.GAME_OVER_2)
                       )
        
        self._current_state_index = 0
        self._current_state = self._states[self._current_state_index]
        
        pygame.display.set_caption("Hamelin")
    
    def main_loop(self):
        self._current_state.entry(self)
        while (not self.input.quit):
            self.input.process()
            self._statemachine_tick()
            pygame.display.flip()
            self.clock.tick(FPS)
        pygame.quit()
    
        
    def _statemachine_tick(self):
        done = self._current_state.do(self)
        if done:
            self._current_state.exit(self)
            self.input.lmb = False
            self.input.rmb = False
            self._current_state_index = (self._current_state_index + 1) % len(self._states)
            self._current_state = self._states[self._current_state_index]
            self._current_state.entry(self)
    
    

class Input(object):
    """Player Input"""
    LMB = 1
    RMB = 3
    WHEEL_UP = 4
    WHEEL_DOWN = 5
    
    def __init__(self):
        pygame.event.set_allowed(None)
        pygame.event.set_allowed(pygame.VIDEORESIZE)
        pygame.event.set_allowed(pygame.QUIT)
        pygame.event.set_allowed(pygame.KEYUP)
        pygame.event.set_allowed(pygame.MOUSEBUTTONDOWN)
        pygame.event.set_allowed(pygame.MOUSEBUTTONUP)
        self.quit = False
        self.lmb = False
        self.rmb = False
        self.wheel_down = False
        self.wheel_up = False
        
    def process(self):
        self.wheel_down = False
        self.wheel_up = False
        
        tmp_mouse_x, tmp_mouse_y = pygame.mouse.get_pos()
        #tmp_mouse_y = RESOLUTION[1] - tmp_mouse_y
        #tmp_mouse_pos[1] = RESOLUTION[1] - tmp_mouse_pos[1]
        self.mouse_pos = glsupport.MapScreenCoordinatesToViewportCoordinates(tmp_mouse_x, tmp_mouse_y)
        
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                self.quit = True
            elif event.type == pygame.VIDEORESIZE:
                glsupport.InitializeOpenGl(event.w, event.h)
            elif event.type == pygame.KEYUP:
                key = event.key
                is_cmd_pressed = (pygame.KMOD_LMETA | pygame.KMOD_RMETA) & event.mod
                
                if ((key == pygame.K_q) and is_cmd_pressed) or (key == pygame.K_ESCAPE):
                    self.quit = True
                    
            elif event.type == pygame.MOUSEBUTTONDOWN:
                if event.button == Input.LMB:
                    self.lmb = True
                elif event.button == Input.RMB:
                    self.rmb = True

                elif event.button == Input.WHEEL_UP:
                    self.wheel_up = True
                elif event.button == Input.WHEEL_DOWN:
                    self.wheel_down = True
                    
            elif event.type == pygame.MOUSEBUTTONUP:
                if event.button == Input.LMB:
                    self.lmb = False
                elif event.button == Input.RMB:
                    self.rmb = False
                    
