from OpenGL.GL import *

import vmath
from vmath import *
import pygame

from pygame.locals import *
import animation
from globalconstants import *

current_screen_width = SCREEN_INIT_WIDTH
current_screen_height = SCREEN_INIT_HEIGHT

default_left_clipping_plane = 0
default_right_clipping_plane = RESOLUTION[0]
default_bottom_clipping_plane = 0
default_top_clipping_plane = RESOLUTION[1]

current_left_clipping_plane = default_left_clipping_plane
current_right_clipping_plane = default_right_clipping_plane
current_bottom_clipping_plane = default_bottom_clipping_plane
current_top_clipping_plane = default_top_clipping_plane

current_scale = SCALING_DEFAULT
current_scale_state = SCALING_STATE_DO_NOTHING
current_scale_aiming = SCALING_DEFAULT


def ClearScreen(r, g, b):
    glClearColor(r ,g, b, 1.0)    
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
        
def MapScreenCoordinatesToViewportCoordinates(x,y):
    
    #first normalize screen data
    r_x = float(x) / current_screen_width
    r_y = 1 - (float(y) / current_screen_height)
        
    # then multiply it to fit into the clipping plane
    r_x = current_left_clipping_plane + (current_right_clipping_plane - current_left_clipping_plane ) * r_x 
    r_y = current_bottom_clipping_plane + (current_top_clipping_plane - current_bottom_clipping_plane ) * r_y
    
    return r_x,r_y    

def InitializeOpenGl(width,height):
    global current_screen_width, current_screen_height
    
    pygame.display.set_mode((width,height),OPENGL|DOUBLEBUF | pygame.RESIZABLE )
    
    current_screen_width = width
    current_screen_height = height
    #current_right_clipping_plane = width
    #current_top_clipping_plane = height
    #set up texturing
    glEnable(GL_TEXTURE_2D)
    glEnable(GL_BLEND)
    # glEnable(GL_DEPTH_TEST)
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
    
    # re-load all textures into open GL memory
    for texture in GL_Texture._all:
        texture.reinitgl()
    
def BeginGLRendering(center_x, center_y):
    """call this to init rendering within one frame"""
    global current_scale, current_scale_state, current_scale_aiming
    global current_left_clipping_plane
    global current_right_clipping_plane
    global current_bottom_clipping_plane
    global current_top_clipping_plane
    
    ClearScreen(0.650980,0.8496240,0.882352) # "water blue"
    
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    
    if current_scale_state == SCALING_STATE_SCALE_UPWARDS:
        current_scale += SCALING_FRAME_STEP
        if current_scale_aiming <= current_scale:
            current_scale_state = SCALING_STATE_DO_NOTHING
            current_scale = current_scale_aiming
                
    if current_scale_state == SCALING_STATE_SCALE_DOWNWARDS:
        current_scale -= SCALING_FRAME_STEP
        if current_scale_aiming >= current_scale:
            current_scale_state = SCALING_STATE_DO_NOTHING
            current_scale = current_scale_aiming                       
    
    current_viewport_width =  (default_right_clipping_plane - default_left_clipping_plane) * current_scale 
    current_viewport_height =  (default_top_clipping_plane - default_bottom_clipping_plane) * current_scale
            
    delta_x = center_x - ( current_viewport_width / 2.0 )
    delta_y = center_y - ( current_viewport_height / 2.0 )
    
    current_left_clipping_plane = (default_left_clipping_plane* current_scale) + delta_x
    current_right_clipping_plane = (default_right_clipping_plane* current_scale) + delta_x
    current_bottom_clipping_plane = (default_bottom_clipping_plane* current_scale) + delta_y
    current_top_clipping_plane = (default_top_clipping_plane* current_scale) + delta_y
    
    glOrtho(current_left_clipping_plane,
             current_right_clipping_plane,
             current_bottom_clipping_plane,
             current_top_clipping_plane,
             -1,
             1)
    
#    glOrtho(default_left_clipping_plane,
#             default_right_clipping_plane,
#             default_bottom_clipping_plane,
#             default_top_clipping_plane,
#             -1,
#             1)
    
    #glOrtho (0,800,0,600,-1,1)
        
def EndGLRendering():
    """ call this to end rendering within one frame"""
    glFlush()


def CreatePositionedTextures(animations, scaling_factor):
    positioned_animations = []
    for animation in animations:
        positioned_animation = []
        positioned_animations.append(positioned_animation)
        for texture in animation:
            positioned_texture = GL_PositionedTexture(texture)
            positioned_texture.set_scaling(scaling_factor)
            positioned_animation.append(positioned_texture)
    return positioned_animations


class GL_Animation(object):
    def __init__(self,textures, scaling_factor):
        self.textures = CreatePositionedTextures(textures, scaling_factor)
        self.animation = 0
        self.frame = 0
        self.pos = Vector(0,0)
        self.red = 255
        self.green = 255
        self.blue = 255
        self.alpha = 255
        self.z = 0
        self.heading = animation.DIRECTION_LEFT
        self.mirrored = False
        
    def GetCurrentAnimationLength(self):
        return len(self.textures[self.animation])
    
    def GetCurrentAnmationFrameSize(self):
        return self.textures[self.animation][self.frame].get_size()
    
    def SetAnimation(self,animation):
        self.animation = animation
        
    def SetFrame(self,frame):
        self.frame = frame 
            
    def render(self):
        glColor4f(self.red,self.green,self.blue,self.alpha)
        
        texture = self.textures[self.animation][self.frame]
        texture.pos_x = self.pos.x
        texture.pos_y = self.pos.y
        texture.pos_z = self.z
        texture.mirror = self.mirrored
        texture.render()
        
        

class GL_Image(object):
    def __init__(self,texture):
        self.texture = GL_PositionedTexture(texture)
        self.pos = Vector(0,0)
        self.red = 255
        self.green = 255
        self.blue = 255
        self.alpha = 255
        self.z = 0
        self.mirrored = False
        
    def render(self):
        glColor4f(self.red,self.green,self.blue,self.alpha)
        
        self.texture.mirror = self.mirrored
        self.texture.pos_x = self.pos.x
        self.texture.pos_y = self.pos.y
        
        self.texture.render()
        
    def changeWidth(self,new_width):
        self.texture.set_width(new_width)
        
    def set_scaling(self, scaling):
        self.texture.set_scaling(scaling)
        
        
class GL_PositionedTexture(object):
    def __init__(self, texture):
        self.texture = texture
        self.set_scaling(1)
        self.pos_x = 0
        self.pos_y = 0
        self.pos_z = 0
        self.mirror = False
        
    def render(self):
        rect = self.get_bounding_rect()
        
        if not is_in_current_viewport(rect):
            return
        
        self.texture.render(rect._x, rect._y, self.pos_z, 
                            rect._w, rect._h,
                            self.mirror)
    
    def get_size(self):
        return (self._width, self._height)
        
    def set_scaling(self, scaling):
        self._scaling = scaling
        self._width = self.texture.width * scaling
        self._height = self.texture.height * scaling
        
    def set_width(self, new_width):
        ratio = float(self.texture.height) / self.texture.width 
        self._width = new_width
        self._height = new_width * ratio
        self._scaling = -1
        
    def get_bounding_rect(self):
        rect = vmath.Rectangle(0, self.pos_y, self._width, self._height)
        rect.set_x_center(self.pos_x)
        return rect

    
def is_in_current_viewport(rect):
    viewport_x = current_left_clipping_plane
    viewport_y = current_bottom_clipping_plane
    viewport_w = current_right_clipping_plane - current_left_clipping_plane
    viewport_h = current_top_clipping_plane - current_bottom_clipping_plane
    
    viewport = vmath.Rectangle(viewport_x, viewport_y, viewport_w, viewport_h)
    
    return viewport.collides_with(rect)
        

def release_all_textures():
    for texture in GL_Texture._all:
        texture.dispose()
    GL_Texture._all = []

class GL_Texture(object):
    _all = []
    
    def __init__(self, path):
        self.textureSurface = pygame.image.load(path)
        self.textureData = pygame.image.tostring(self.textureSurface, "RGBA", 1)
        
        GL_Texture._all.append(self)
        
        self.reinitgl()
        
        self.width = self.textureSurface.get_width();
        self.height = self.textureSurface.get_height();
        
    def dispose(self):
        glDeleteTextures([self.texture])
        
    def reinitgl(self):
        """has to be called whenever opengl has been reinitialized"""
        self.texture = glGenTextures(1)
        glBindTexture(GL_TEXTURE_2D, self.texture)
        
        #glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
        #glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST)
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST)
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP)
        glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP)
        
        glTexImage2D(
                     GL_TEXTURE_2D, 0, 
                     GL_RGBA, 
                     self.textureSurface.get_width(), 
                     self.textureSurface.get_height(), 0, 
                     GL_RGBA, GL_UNSIGNED_BYTE, 
                     self.textureData) 
        
        

    def render(self, x, y, z, w, h, mirror):
        glPushMatrix()
        
        glBindTexture(GL_TEXTURE_2D, self.texture)
        
        glBegin(GL_QUADS)
        
        if mirror:
            x1 = 1.0
            x2 = 0.0
        else:
            x1 = 0.0
            x2 = 1.0
             
        glTexCoord2f(x1, 0.0); glVertex3f(x, y, z)
        glTexCoord2f(x2, 0.0); glVertex3f(x + w, y , z)
        glTexCoord2f(x2, 1.0); glVertex3f(x + w, y + h, z)
        glTexCoord2f(x1, 1.0); glVertex3f(x, y + h, z)
        glEnd();
        glPopMatrix()
        
        
        