from __future__ import division
import math

EPSILON = 0.0001

class Vector(object):
    def __init__(self, x, y):
        self.x = x
        self.y = y
        
    def __repr__(self):
        return "Vector(%s, %s)" % (repr(self.x), repr(self.y))
        
    def __str__(self):
        return "(%s, %s)" % (str(self.x), str(self.y))
        
    def __abs__(self):
        return (self.x**2 + self.y**2)**(1/2)
    
    def angle(self):
        if (self.y < 0):
            if (self.x < 0):
                # 3rd quadrant
                return math.acos(-self.x/self.abs()) + math.pi
            else:
                # 4th quadrant
                return 2*math.pi - math.acos(self.x/self.abs())
        else:
            if (self.x < 0):
                # 2nd quadrant
                return math.pi - math.acos(-self.x/self.abs())
            else:
                # 1st quadrant
                return math.acos(self.x/self.abs())
            
    def __mul__(self, other):
        return Vector(self.x*other, self.y*other)
    
    def __add__(self, other):
        return Vector(self.x+other.x, self.y+other.y)
    
    def __sub__(self, other):
        return Vector(self.x-other.x, self.y-other.y)
    
    def __div__(self, other):
        return Vector(self.x/other, self.y/other)
    
    __truediv__ = __div__
    
    def __cmp__(self,other):
        if (self.x == other.x and self.y == other.y) == True:
            return True
        else:
            return False
    
    def get_vector_of_length(self, length):
        return self / abs(self) * length        
    
    def rotate(self, angle):
        return Vector(self.x * math.cos(angle) - self.y * math.sin(angle),
                      self.y * math.cos(angle) + self.x * math.sin(angle))
        
    def to_int_tuple(self):
        '''returns an int tuple (x,y) that can be used as point in pygame drawing operations'''
        return (int(self.x), int(self.y))


class Rectangle(object):
    def __init__(self, x, y, w, h):
        self._x = x
        self._y = y
        self._w = w
        self._h = h
    
    def set_x_center(self, x_center):
        self._x = x_center - (self._w / 2.0)
        
    def collides_with(self, other):
        return (self._overlaps_1d(self._x, self._x + self._w, other._x, other._x + other._w) 
                and
                self._overlaps_1d(self._y, self._y + self._h, other._y, other._y + other._h)
                )
    
    def _overlaps_1d(self, a1, a2, b1, b2):
        return (
                (a1 >= b1 and a1 <= b2) 
                or 
                (a2 >= b1 and a2 <= b2)
                or
                (b1 >= a1 and b1 <= a2) 
                or 
                (b2 >= a1 and b2 <= a2)
                )
    
    